@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GNOME_SETTINGS_PLUGIN_H__
@@ -97,7 +95,7 @@ plugin_name##_plugin_init (PluginName##Plugin *plugin)                         \
 {                                                                              \
         plugin->priv = G_TYPE_INSTANCE_GET_PRIVATE ((plugin),                  \
                 plugin_name##_plugin_get_type(), PluginName##PluginPrivate);   \
-        g_debug ("PluginName## initializing");                                 \
+        g_debug (#PluginName " initializing");                                 \
         plugin->priv->manager = plugin_name##_manager_new ();                  \
 }                                                                              \
                                                                                \
