@@ -13,8 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -499,7 +498,6 @@ run_custom_command (GdkDevice              *device,
                     CustomCommand           command)
 {
         GSettings *settings;
-        GError *error = NULL;
         char *cmd;
         char *argv[7];
         int exit_status;
@@ -532,25 +530,15 @@ run_custom_command (GdkDevice              *device,
         g_free (out);
 
         rc = g_spawn_sync (g_get_home_dir (), argv, NULL, G_SPAWN_SEARCH_PATH,
-                           NULL, NULL, NULL, NULL, &exit_status, &error);
+                           NULL, NULL, NULL, NULL, &exit_status, NULL);
 
-        if (rc == FALSE) {
-                g_warning ("Couldn't execute command '%s', verify that this is a valid command: %s", cmd, error->message);
-                g_clear_error (&error);
-        }
+        if (rc == FALSE)
+                g_warning ("Couldn't execute command '%s', verify that this is a valid command.", cmd);
 
         g_free (argv[0]);
         g_free (argv[4]);
 
-        if (!g_spawn_check_exit_status (exit_status, &error)) {
-                if (g_error_matches (error, G_SPAWN_EXIT_ERROR, 1)) {
-                        g_clear_error (&error);
-                        return TRUE;
-                }
-                g_clear_error (&error);
-        }
-
-        return FALSE;
+        return (exit_status == 1);
 }
 
 GList *
@@ -588,10 +576,37 @@ get_disabled_devices (GdkDeviceManager *manager)
         return ret;
 }
 
-void
-xdevice_close (XDevice *xdevice)
+const char *
+xdevice_get_wacom_tool_type (int deviceid)
 {
-    gdk_error_trap_push ();
-    XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
-    gdk_error_trap_pop_ignored();
+        unsigned long nitems, bytes_after;
+        unsigned char *data = NULL;
+        Atom prop, realtype, tool;
+        GdkDisplay *display;
+        int realformat, rc;
+        const gchar *ret = NULL;
+
+        gdk_error_trap_push ();
+
+        display = gdk_display_get_default ();
+        prop = gdk_x11_get_xatom_by_name ("Wacom Tool Type");
+
+        rc = XIGetProperty (GDK_DISPLAY_XDISPLAY (display),
+                            deviceid, prop, 0, 1, False,
+                            XA_ATOM, &realtype, &realformat, &nitems,
+                            &bytes_after, &data);
+
+        gdk_error_trap_pop_ignored ();
+
+        if (rc != Success || nitems == 0)
+                return NULL;
+
+        if (realtype == XA_ATOM) {
+                tool = *((Atom*) data);
+                ret = gdk_x11_get_xatom_name (tool);
+        }
+
+        XFree (data);
+
+        return ret;
 }
